<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

require_once MOONY_ROOT_DIR . '/Moony/View.php';

/**
 * アクションの基底クラスです。
 * 全てのアクションクラスはこのクラスを継承して実装してください。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Action
{
    /**
     * Moonyのインスタンス（動作設定）
     * @var object
     */
    var $moony;

    /**
     * Moony_Requestのインスタンス
     * @var object
     */
    var $request;

    /**
     * Moony_Sessionのインスタンス
     * @var object
     */
    var $session;

    /**
     * Moony_Cacheのインスタンス
     * @var object
     */
    var $cache;

    /**
     * Moony_Validatorのインスタンス
     * @var object
     */
    var $validator;

    /**
     * アクションの処理を実行します。
     * 実装時にオーバーライドしてください。
     * オーバーライドしない場合、テンプレートの処理結果を出力します。
     *
     * @access public
     */
    function execute()
    {
        $this->render();
    }

    /**
     * テンプレートの処理結果を取得します。
     * $templateが設定されていない場合、
     * 使用するテンプレートをアクションパスを元に決定します。
     * テンプレートファイルが存在しない場合、falseを返します。
     * テンプレートからアクションクラスのpublicなメンバ変数には
     * <?php h($this->foo); ?>とアクセスすることができます。
     * $local_varsが設定されている場合、値は展開されて
     * <?php h($foo); ?>のように利用することができます。
     *
     * @access public
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @return string|bool テンプレートの処理結果
     */
    function fetch($template = null, $local_vars = null)
    {
        if (is_null($template)) {
            // if not defined: $template
            $action_path = $this->request->getActionPath();
            $template = strtolower($action_path) . '.php';
        }

        // fetch the template
        $view =& new Moony_View($this);
        $result = $view->fetch($template, $local_vars);

        return $result;
    }

    /**
     * テンプレートの処理結果を出力します。
     * $templateが設定されていない場合、
     * 使用するテンプレートをアクションパスを元に決定します。
     * テンプレートファイルが存在しない場合、
     * HTTPのステータスで404を送出して処理を終了します。
     * テンプレートからアクションクラスのpublicなメンバ変数には
     * <?php h($this->foo); ?>とアクセスすることができます。
     * $local_varsが設定されている場合、値は展開されて
     * <?php h($foo); ?>のように利用することができます。
     * $convert_encodingがtrueで出力エンコーディングが指定されている場合、
     * 出力エンコーディングに変換された内容が出力されます。
     *
     * @access public
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @param bool $convert_encoding エンコーディング変換を行うかどうか
     */
    function render(
        $template = null, $local_vars = null, $convert_encoding = true)
    {
        $result = $this->fetch($template, $local_vars);
        if ($result === false) {
            // not found
            ob_end_clean();
            $this->sendStatus(404);
            exit;
        }
        $this->renderText($result, $convert_encoding);
    }

    /**
     * テキストを出力します。
     * $convert_encodingがtrueで出力エンコーディングが指定されている場合、
     * 出力エンコーディングに変換された内容が出力されます。
     *
     * @access public
     * @param string $text 出力するテキスト
     * @param bool $convert_encoding エンコーディング変換を行うかどうか
     */
    function renderText($text, $convert_encoding = true)
    {
        if ($convert_encoding) {
            if (!is_null($this->moony->output_encoding)) {
                $text = mb_convert_encoding($text,
                    $this->moony->output_encoding,
                    $this->moony->internal_encoding);
            }
        }
        print($text);
    }

    /**
     * 値をJSON形式に変換して出力します。
     * このメソッドを正常に使用するためには、
     * JSONエクステンションまたはJsphonが利用可能である必要があります。
     * いずれもインストールされていない場合、警告を発生、falseを返します。
     * $mime_typeはHTTPヘッダのContent-Typeとして送出されます。
     *
     * @access public
     * @param mixed $value JSONに変換して出力する値
     * @param string $mime_type 出力時に使用されるMIMEタイプ
     * @see http://pecl.php.net/package/json
     * @see http://www.hawklab.jp/jsonencoder/
     * @see http://www.rfc-editor.org/rfc/rfc4627.txt
     */
    function renderJson($value, $mime_type = 'application/json')
    {
        // to UTF-8
        if ($this->moony->internal_encoding != 'UTF-8') {
            mb_convert_variables(
                'UTF-8', $this->moony->internal_encoding, $value);
        }

        $json = null;
        if (function_exists('json_encode')) {
            // use JSON extension
            $json = json_encode($value);
        } else if (class_exists('Jsphon')) {
            // use Jsphon
            $json = Jsphon::encode($value);
        } else {
            trigger_error(
                'Not loaded: JSON extension or Jsphon', E_USER_WARNING);
            return false;
        }

        if (!is_null($mime_type)) {
            header("Content-Type: {$mime_type}");
        }
        header('Content-Length: ' . strlen($json));
        print($json);
    }

    /**
     * 任意の画像を出力します。
     * 指定されたファイルが存在しないか、
     * ファイルを読み込むことができない場合にfalseを返します。
     * （使用例: $this->renderImage('/path/to/file', 'image/jpeg');）
     *
     * @access public
     * @param string $path 描画するファイル
     * @param string $content_type MIMEタイプ（image/jpegなど）
     * @param int $buffer_size ファイル読み込みサイズ
     * @return null|bool ファイルが存在しない場合false
     */
    function renderImage($path, $content_type, $buffer_size = 4096)
    {
        $path = realpath($path);
        if (!file_exists($path) || !is_file($path)) {
            return false;
        }
        $fp = fopen($path, 'rb');
        if ($fp === false) {
            return false;
        }

        header("Content-Type: {$content_type}");
        header('Content-Disposition: inline');
        header('Content-Length: ' . filesize($path));

        while (!feof($fp)) {
            print(fread($fp, $buffer_size));
        }
        fclose($fp);
    }

    /**
     * リダイレクト処理を行い、処理を終了します。
     * $statusが設定されている場合、HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param string $url リダイレクト先のURL
     * @param int $status ステータスコード
     */
    function redirect($url, $status = null)
    {
        if (ob_get_length() !== false) {
            ob_end_clean();
        }
        if (!is_null($status)) {
            $this->sendStatus($status);
        }
        header("Location: {$url}");
        exit;
    }

    /**
     * 指定された名称のアクションにリダイレクト処理を行い、
     * 処理を終了します。
     * $action_paramsが設定されていた場合、
     * URLの末尾にPATH_INFOとして付加、パラメータとして渡します。
     * $statusが設定されている場合、HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param string $action_name リダイレクト先のアクション名
     * @param array $action_params アクションに渡すパラメータの配列
     * @param int $status ステータスコード
     */
    function redirectAction(
        $action_name, $action_params = null, $status = null)
    {
        $action_path = strtolower(strtr($action_name, '_', '/'));
        $url = $this->request->getHostUrl()
             . $this->request->getActionBase()
             . '/' . $action_path;
        if (is_array($action_params)) {
            foreach ($action_params as $param) {
                $url .= '/' . urlencode($param);
            }
        }
        $this->redirect($url, $status);
    }

    /**
     * 検証エラーが存在する場合、
     * テンプレートの処理結果を出力して処理を終了します。
     * $templateが設定されていない場合、
     * 使用するテンプレートをアクションパスを元に決定します。
     * テンプレートファイルが存在しない場合、
     * HTTPのステータスで404を送出して処理を終了します。
     * テンプレートからアクションクラスのpublicなメンバ変数には
     * <?php h($this->foo); ?>とアクセスすることができます。
     * $local_varsが設定されている場合、値は展開されて
     * <?php h($foo); ?>のように利用することができます。
     * $convert_encodingがtrueで出力エンコーディングが指定されている場合、
     * 出力エンコーディングに変換された内容が出力されます。
     * 検証エラーが存在しない場合、何も処理されません。
     *
     * @access public
     * @param string $template テンプレートファイル
     * @param array $local_vars ローカル変数として展開する値の連想配列
     * @param bool $convert_encoding エンコーディング変換を行うかどうか
     */
    function renderOnError(
        $template = null, $local_vars = null, $convert_encoding = true)
    {
        if (is_null($this->validator)) {
            return false;
        }
        if ($this->validator->hasError()) {
            $this->render($template, $local_vars, $convert_encoding);
            exit;
        }
    }

    /**
     * 検証エラーが存在する場合、
     * リダイレクト処理を行って処理を終了します。
     * $statusが設定されている場合、HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param string $url リダイレクト先のURL
     * @param int $status ステータスコード
     */
    function redirectOnError($url, $status = null)
    {
        if (is_null($this->validator)) {
            return false;
        }
        if ($this->validator->hasError()) {
            $this->redirect($url, $status);
            exit;
        }
    }

    /**
     * 検証エラーが存在する場合、
     * 指定された名称のアクションにリダイレクト処理を行い、
     * 処理を終了します。
     * $action_paramsが設定されていた場合、
     * URLの末尾にPATH_INFOとして付加、パラメータとして渡します。
     * $statusが設定されている場合、HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param string $action_name リダイレクト先のアクション名
     * @param array $action_params アクションに渡すパラメータの配列
     * @param int $status ステータスコード
     */
    function redirectActionOnError(
        $action_name, $action_params = null, $status = null)
    {
        if (is_null($this->validator)) {
            return false;
        }
        if ($this->validator->hasError()) {
            $this->redirectAction($action_name, $action_params, $status);
            exit;
        }
    }

    /**
     * HTTPのステータスヘッダを送出します。
     *
     * @access public
     * @param int $status ステータスコード
     * @see http://www.rfc-editor.org/rfc/rfc1945.txt
     * @see http://www.rfc-editor.org/rfc/rfc2616.txt
     */
    function sendStatus($status)
    {
        // HTTP/1.1?
        $protocol = $this->request->getServer('SERVER_PROTOCOL');
        $is_http_11 = (strpos($protocol, '1.1') !== false);
        // messages
        $messages = array(
            // Informational 1xx
            100 => 'Continue',
            101 => 'Switching Protocols',
            // Success 2xx
            200 => 'OK',
            201 => 'Created',
            202 => 'Accepted',
            203 => 'Non-Authoritative Information',
            204 => 'No Content',
            205 => 'Reset Content',
            206 => 'Partial Content',
            // Redirection 3xx
            300 => 'Multiple Choices',
            301 => 'Moved Permanently',
            302 => 'Found',  // 1.1
            303 => 'See Other',
            304 => 'Not Modified',
            305 => 'Use Proxy',
            // 306 is no longer used but still reserved
            307 => 'Temporary Redirect',
            // Client Error 4xx
            400 => 'Bad Request',
            401 => 'Unauthorized',
            402 => 'Payment Required',
            403 => 'Forbidden',
            404 => 'Not Found',
            405 => 'Method Not Allowed',
            406 => 'Not Acceptable',
            407 => 'Proxy Authentication Required',
            408 => 'Request Timeout',
            409 => 'Conflict',
            410 => 'Gone',
            411 => 'Length Required',
            412 => 'Precondition Failed',
            413 => 'Request Entity Too Large',
            414 => 'Request-URI Too Long',
            415 => 'Unsupported Media Type',
            416 => 'Requested Range Not Satisfiable',
            417 => 'Expectation Failed',
            // Server Error 5xx
            500 => 'Internal Server Error',
            501 => 'Not Implemented',
            502 => 'Bad Gateway',
            503 => 'Service Unavailable',
            504 => 'Gateway Timeout',
            505 => 'HTTP Version Not Supported',
            509 => 'Bandwidth Limit Exceeded'
        );
        if (isset($messages[$status])) {
            if (!$is_http_11) {
                $messages[302] = 'Moved Temporarily';
            }
            $http_version = $is_http_11 ? '1.1' : '1.0';
            header("HTTP/{$http_version} {$status} {$messages[$status]}");
        }
    }

    /**
     * 任意のファイルをダウンロードさせます。
     * $nameが設定されていない場合、元のファイル名を使用します。
     * 指定されたファイルが存在しないか、
     * ファイルを読み込むことができない場合にfalseを返します。
     * 使用例: $this->sendFile('/path/to/foo.zip');
     *
     * @access public
     * @param string $path ダウンロードするファイルのパス
     * @param string $name ダウンロードするファイルにつける名前
     * @param int $buffer_size ファイル読み込み時のバッファサイズ
     * @return null|bool ファイルが存在しない場合false
     */
    function sendFile($path, $name = null, $buffer_size = 4096)
    {
        $path = realpath($path);
        if ($path === false || !is_file($path)) {
            return false;
        }
        if (is_null($name)) {
            // original name
            $name = basename($path);
        }

        $fp = fopen($path, 'rb');
        if ($fp === false) {
            return false;
        }

        header('Content-Type: application/octet-stream');
        header("Content-Disposition: attachment; filename=\"{$name}\"");
        header('Content-Length: ' . filesize($path));

        while (!feof($fp)) {
            echo fread($fp, $buffer_size);
        }
        fclose($fp);
    }

    /**
     * アップロードされたファイルの保存を行います。
     * 保存先のディレクトリは前もって用意されている必要があります。
     * なお、配列を用いた複数ファイルのアップロードには対応していません。
     *
     * @access public
     * @param string $name アップロードファイル名
     * @param string $path 保存先のパス
     * @param int $mode 保存先のファイルのモード
     * @param bool $delete 保存後に一時ファイルを削除するかどうか
     * @return bool アップロードに成功したかどうか
     */
    function saveFile($name, $path, $mode = 0666, $delete = false)
    {
        $file_info = $this->request->getFile($name);
        if (is_null($file_info) ||
            !is_uploaded_file($file_info['tmp_name'])) {
            // not uploaded
            return false;
        }
        $dir = dirname($path);
        if (!file_exists($dir) || !is_dir($dir)) {
            // directory not found
            return false;
        }
        if ($file_info['error'] != UPLOAD_ERR_OK) {
            // upload error
            return false;
        }
        if (move_uploaded_file($file_info['tmp_name'], $path)) {
            // move uploaded file
            chmod($path, $mode);
            if ($delete) {
                unlink($file_info['tmp_name']);
            }
            return true;
        }
        return false;
    }

    /**
     * ワンタイムトークンのチェックを行います。
     * リクエストパラメータとして渡された値と、
     * セッションに保存されていた値が一致する場合のみ妥当と判断します。
     *
     * @access public
     * @return bool 妥当なワンタイムトークンかどうか
     */
    function checkToken()
    {
        // remove from the session
        $saved = $this->session->remove('_moony_token');
        $received = $this->request->get('_moony_token');
        if (strlen($saved) > 0 && strlen($received) > 0) {
            return $saved === $received;
        }
        return false;
    }
}
?>
