<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2007 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

/**
 * 簡単なキャッシュ機能を実現するためのクラスです。
 * キャッシュID単位でキャッシュファイルを生成、
 * 有効期間を設けてキャッシュの管理を行います。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Cache
{
    /**
     * キャッシュファイルを格納するディレクトリ
     * @var string
     */
    var $_cache_dir;

    /**
     * 出力キャッシュのキャッシュID
     * @var string
     */
    var $_output_id;

    /**
     * 出力キャッシュの有効期間（秒）
     * @var int
     */
    var $_output_lifetime;

    /**
     * コンストラクタです。
     *
     * @access public
     * @param string $cache_dir キャッシュファイルを格納するディレクトリ
     */
    function Moony_Cache($cache_dir)
    {
        $this->setCacheDir($cache_dir);
    }

    /**
     * キャッシュの内容を取得します。
     * キャッシュファイルが存在しない場合、falseを返します。
     * キャッシュファイルが存在し、有効期限が切れている場合は
     * キャッシュファイルを削除した上で、falseを返します。
     *
     * @access public
     * @param string $id キャッシュID
     * @return mixed キャッシュされている内容
     */
    function get($id)
    {
        $value = $this->_getCache($id);
        if ($value === false) {
            return false;
        }
        return $value['content'];
    }

    /**
     * キャッシュを行います。
     * キャッシュに失敗した場合、falseを返します。
     *
     * @access public
     * @param string $id キャッシュID
     * @param mixed $content キャッシュする内容
     * @param int $lifetime キャッシュの有効期間（秒）
     */
    function save($id, $content, $lifetime = 3600)
    {
        $path = $this->_getPath($id);
        $expiration_time = time() + $lifetime;

        $cache = array(
            'path' => $path,
            'expiration_time' => $expiration_time,
            'content' => $content
        );

        // open in read/write mode
        $fp = fopen($path, 'a+');
        if ($fp === false) {
            return false;
        }
        flock($fp, LOCK_EX);
        fseek($fp, 0);
        if (fwrite($fp, serialize($cache)) === false) {
            return false;
        }
        fclose($fp);
        chmod($path, 0666);
    }

    /**
     * キャッシュが存在するかどうかを調べます。
     *
     * @access public
     * @param string $id キャッシュID
     * @return bool キャッシュが存在するかどうか
     */
    function exists($id)
    {
        $cache = $this->_getCache($id);
        return $cache !== false;
    }

    /**
     * キャッシュファイルを削除します。
     * キャッシュファイルが存在しない場合、falseを返します。
     *
     * @access public
     * @param string $id 削除するキャッシュID
     * @return mixed 削除したキャッシュの値
     */
    function remove($id)
    {
        $cache = $this->_getCache($id);
        if ($cache === false) {
            return false;
        }
        if (!unlink($cache['path'])) {
            // unable to remove a cache file?
            $this->save($id, $cache['content'], -1);
        }
        return $cache['content'];
    }

    /**
     * 期限切れのキャッシュファイルを全て削除します。
     *
     * @access public
     */
    function clean()
    {
        $cwd = getcwd();
        chdir($this->_cache_dir);
        $cache_files = glob('moony_cache_*');
        foreach ($cache_files as $cache_file) {
            $cache = $this->_getFile($cache_file);
            if ($cache === false) {
                // ignore error
                continue;
            }
            $cache = unserialize($cache);
            if ($cache['expiration_time'] < time()) {
                if (!unlink($cache_file)) {
                    // unable to remove a cache file?
                    $this->save($id, $cache['content'], -1);
                }
            }
        }
        chdir($cwd);
    }

    /**
     * キャッシュの出力を試みます。
     * 指定されたIDのキャッシュが既に存在する場合、
     * キャッシュされていた内容を出力してtrueを返します。
     * キャッシュされていなかった場合、
     * 出力キャッシュを開始してfalseを返します。
     * 送出されたヘッダはキャッシュされないことに注意してください。
     * なお、複数の出力キャッシュを同時に扱うことはできません。
     *
     * @access public
     * @param string $id キャッシュID
     * @param int $lifetime 出力キャッシュの有効期間（秒）
     * @return bool 出力キャッシュが開始されたかどうか
     */
    function outputStart($id, $lifetime = 3600)
    {
        $cache = $this->_getCache($id);
        if ($cache !== false) {
            // has already cached
            echo $cache['content'];
            return true;
        }

        $this->_output_id = $id;
        $this->_output_lifetime = $lifetime;

        ob_start();
        ob_implicit_flush(false);
        return false;
    }

    /**
     * 出力キャッシュを終了します。
     * 出力内容をキャッシュファイルに保存した上で出力します。
     * 送出されたヘッダはキャッシュされないことに注意してください。
     *
     * @access public
     */
    function outputEnd()
    {
        if (is_null($this->_output_id)) {
            trigger_error('Not started: output caching', E_USER_WARNING);
            return;
        }

        $content = ob_get_contents();
        ob_end_clean();
        $this->save($this->_output_id, $content, $this->_output_lifetime);
        echo $content;

        // clean
        $this->_output_id = null;
        $this->_output_lifetime = null;
    }

    /**
     * キャッシュファイルを格納するディレクトリを設定します。
     * ディレクトリが存在しない場合、作成を試みます。
     *
     * @access public
     * @param string $cache_dir キャッシュファイルを格納するディレクトリ
     */
    function setCacheDir($cache_dir)
    {
        $cache_dir = realpath($cache_dir);
        if ($cache_dir === false) {
            if (mkdir($cache_dir)) {
                chmod($cache_dir, 0777);
            } else {
                // unable to create a cache directory
                trigger_error(
                    'Unable to create: cache_dir', E_USER_WARNING);
            }
        }
        $this->_cache_dir = $cache_dir;
    }

    /**
     * キャッシュの情報を取得します。
     * キャッシュファイルが存在しない場合、falseを返します。
     * キャッシュファイルが存在し、有効期限が切れている場合は
     * キャッシュファイルを削除した上で、falseを返します。
     *
     * @access protected
     * @param string $id キャッシュID
     * @return array キャッシュ情報の連想配列
     */
    function _getCache($id)
    {
        $path = $this->_getPath($id);
        if (!file_exists($path) || is_dir($path)) {
            return false;
        }
        $cache = $this->_getFile($path);
        if ($cache === false) {
            return false;
        }
        $cache = unserialize($cache);
        if ($cache['expiration_time'] < time()) {
            unlink($path);
            return false;
        }
        return $cache;
    }

    /**
     * キャッシュファイルのパスを取得します。
     *
     * @access protected
     * @param string $id キャッシュID
     * @return string キャッシュファイルのパス
     */
    function _getPath($id)
    {
        $file = 'moony_cache_' . md5($id);
        return $this->_cache_dir . DIRECTORY_SEPARATOR . $file;
    }

    /**
     * ファイルの内容を共有ロックをかけた上で取得します。
     * 読み込みに失敗した場合、falseを返します。
     *
     * @access protected
     * @param string $path 読み込むファイルのパス
     * @return string ファイルの内容
     */
    function _getFile($path)
    {
        $fp = fopen($path, 'r');
        if ($fp === false) {
            return false;
        }
        flock($fp, LOCK_SH);
        $data = '';
        while (feof($fp) === false) {
            $data .= fread($fp, 4096);
        }
        fclose($fp);
        return $data;
    }
}
?>
